﻿namespace WarmlyLocomotove
{
    public partial class FormLocomotiveAdditional : Form
    {
        /// <summary>
        /// Объект от класса отрисовки локомотива
        /// </summary>
        private DrawningLocomotive _locomotive;
        /// <summary>
        /// Объект от параметризованного класса с параметрами типа сущности и интерфейса отрисовки дополнительных частей
        /// </summary>
        private AdvancedObjectGeneric<EntityLocomotive,IDrawningAdditionalElements> AdvancedObject = new AdvancedObjectGeneric<EntityLocomotive, IDrawningAdditionalElements>();
        /// <summary>
        /// Конструктор
        /// </summary>
        public FormLocomotiveAdditional()
        {
            InitializeComponent();
        }
        /// <summary>
        /// Метод отрисовки локомотива
        /// </summary>
        private void Draw()
        {
            Bitmap bmp = new(pictureBoxLocomotive.Width, pictureBoxLocomotive.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _locomotive?.DrawTransport(gr);
            pictureBoxLocomotive.Image = bmp;
        }
        /// <summary>
        /// Метод обработки нажатия на кнопки перемещения
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonMove_Click(object sender, EventArgs e)
        {
            //получаем имя кнопки
            string name = ((Button)sender)?.Name ?? string.Empty;
            switch (name)
            {
                case "buttonUp":
                    _locomotive?.MoveLocomotive(Direction.Up);
                    break;
                case "buttonDown":
                    _locomotive?.MoveLocomotive(Direction.Down);
                    break;
                case "buttonLeft":
                    _locomotive?.MoveLocomotive(Direction.Left);
                    break;
                case "buttonRight":
                    _locomotive?.MoveLocomotive(Direction.Right);
                    break;
            }
            Draw();
        }
        /// <summary>
        /// Изменение размеров формы
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PictureBoxLocomotive_Resize(object sender, EventArgs e)
        {
            _locomotive?.ChangeBorders(pictureBoxLocomotive.Width, pictureBoxLocomotive.Height);
            Draw();
        }
        /// <summary>
        /// Метод обработки нажатия на кнопку "Сгенерировать сущности"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonGenerateEntities_Click(object sender, EventArgs e)
        {
            Random rnd = new();
            for (int i = 0; i < 10; i++)
            {
                bool IsAdvancedEntity = Convert.ToBoolean(rnd.Next(0, 2));
                EntityLocomotive Entity;
                if (!IsAdvancedEntity)
                {
                    Entity = new EntityLocomotive(rnd.Next(100, 300), rnd.Next(1000, 2000), Color.FromArgb(rnd.Next(0, 256), rnd.Next(0, 256), rnd.Next(0, 256)));
                }
                else
                {
                    Entity = new EntityWarmlyLocomotive(rnd.Next(100, 300), rnd.Next(1000, 2000),
                    Color.FromArgb(rnd.Next(0, 256), rnd.Next(0, 256), rnd.Next(0, 256)),
                    Color.FromArgb(rnd.Next(0, 256), rnd.Next(0, 256), rnd.Next(0, 256)),
                    Convert.ToBoolean(rnd.Next(0, 2)),
                    Convert.ToBoolean(rnd.Next(0, 2)));
                }
                AdvancedObject.AddEntity(Entity);
            }
        }
        /// <summary>
        /// Метод обработки нажатия на кнопку "Сгенерировать доп. элементы"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonGenerateAdditionalElementsSetup_Click(object sender, EventArgs e)
        {
            Random rnd = new();
            for (int i = 0; i < 10; i++)
            {
                int AdditionalElementSetupType = rnd.Next(0, 3);
                switch (AdditionalElementSetupType)
                {
                    case 0:
                        AdvancedObject.AddAdditionalElementSetup(new DrawningWheels());
                        break;
                    case 1:
                        AdvancedObject.AddAdditionalElementSetup(new DrawningRectOrnament());
                        break;
                    case 2:
                        AdvancedObject.AddAdditionalElementSetup(new DrawningEllipseOrnament());
                        break;
                }
            }
        }
        /// <summary>
        /// Метод обработки нажатия на кнопку "Сгенерировать объект"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonShowGeneratedObject_Click(object sender, EventArgs e)
        {
            Random rnd = new();
            _locomotive = AdvancedObject.GeneratedObject();
            _locomotive.SetPosition(rnd.Next(10, 100), rnd.Next(10, 100), pictureBoxLocomotive.Width, pictureBoxLocomotive.Height);
            _locomotive.AdditionalElements.WheelsNum = 2;
            _locomotive.AdditionalElements.WheelsNum = (int)numericUpDownWheelsNumber.Value;
            Draw();
        }
    }
}
