﻿namespace WarmlyLocomotove
{
    /// <summary>
    /// Форма создания объекта
    /// </summary>
    public partial class FormLocomotiveConfig : Form
    {
        /// <summary>
        /// Переменная - выбранный локомотив
        /// </summary>
        DrawningLocomotive _locomotive = null;
        /// <summary>
        /// Делегат
        /// </summary>
        /// <param name="locomotive"></param>
        public delegate void Action(DrawningLocomotive locomotive);
        /// <summary>
        /// Событие
        /// </summary>
        private event Action EventAddLocomotive;
        /// <summary>
        /// Конструктор
        /// </summary>
        public FormLocomotiveConfig()
        {
            InitializeComponent();
            panelAqua.MouseDown += PanelColor_MouseDown;
            panelBlack.MouseDown += PanelColor_MouseDown;
            panelBlue.MouseDown += PanelColor_MouseDown;
            panelGreen.MouseDown += PanelColor_MouseDown;
            panelPink.MouseDown += PanelColor_MouseDown;
            panelRed.MouseDown += PanelColor_MouseDown;
            panelWhite.MouseDown += PanelColor_MouseDown;
            panelYellow.MouseDown += PanelColor_MouseDown;
            //Лямбда-выражение для закрытия окна
            buttonCancel.Click += (sender, e) => Close();
        }
        /// <summary>
        /// Отрисовка локомотива
        /// </summary>
        private void DrawLocomotive()
        {
            Bitmap bmp = new(pictureBoxObject.Width, pictureBoxObject.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _locomotive?.SetPosition(5, 5, pictureBoxObject.Width, pictureBoxObject.Height);
            _locomotive?.DrawTransport(gr);
            pictureBoxObject.Image = bmp;
        }
        /// <summary>
        /// Добавление события
        /// </summary>
        /// <param name="ev"></param>
        public void AddEvent(Action ev)
        {
            if (EventAddLocomotive == null)
            {
                EventAddLocomotive = new Action(ev);
            }
            else
            {
                EventAddLocomotive += ev;
            }
        }
        /// <summary>
        /// Передаём информацию при нажатии на Label
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LabelObject_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Label).DoDragDrop((sender as Label).Name, DragDropEffects.Move | DragDropEffects.Copy);
        }
        /// <summary>
        /// Проверка получаемой информации
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PanelObject_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.Text) || e.Data.GetDataPresent(typeof(IDrawningAdditionalElements)))
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }
        /// <summary>
        /// Действие при приёме получаемой информации
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PanelObject_DragDrop(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.Text))
            {
                _locomotive = new DrawningWarmlyLocomotive((int)numericUpDownSpeed.Value, (int)numericUpDownWeight.Value, Color.White, 160, 85, Color.Black, checkBoxHasPipe.Checked, checkBoxHasFuelTank.Checked);
                switch (e.Data.GetData(DataFormats.Text).ToString())
                {
                    case "labelSimpleObject":
                        _locomotive = new DrawningLocomotive((int)numericUpDownSpeed.Value, (int)numericUpDownWeight.Value, Color.White);
                        break;
                    case "labelModifiedObject":
                        _locomotive = new DrawningWarmlyLocomotive((int)numericUpDownSpeed.Value, (int)numericUpDownWeight.Value, Color.White, 160, 85, Color.Black, checkBoxHasPipe.Checked, checkBoxHasFuelTank.Checked);
                        break;
                }
            }
            else if (_locomotive != null && e.Data.GetDataPresent(typeof(IDrawningAdditionalElements)))
            {
                var ornament = e.Data.GetData(typeof(IDrawningAdditionalElements));
                _locomotive.AdditionalElements = (IDrawningAdditionalElements)ornament;
            }
            _locomotive.AdditionalElements.WheelsNum = (int)numericUpDownWheelsNumber.Value;
            DrawLocomotive();
        }
        /// <summary>
        /// Отправляем цвет с панели
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PanelColor_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Panel).DoDragDrop((sender as Panel).BackColor, DragDropEffects.Move | DragDropEffects.Copy);
        }
        /// <summary>
        /// Проверка получаемой информации
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LabelColor_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(typeof(Color)))
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect= DragDropEffects.None;
            }
        }
        /// <summary>
        /// Принимаем основной цвет
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LabelColor_DragDrop(object sender, DragEventArgs e)
        {
            _locomotive.Locomotive.BodyColor = (Color)e.Data.GetData(typeof(Color));
            DrawLocomotive();
        }
        /// <summary>
        /// Принимаем дополнительный цвет
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LabelAdditionalColor_DragDrop(object sender, DragEventArgs e)
        {
            if (_locomotive.Locomotive is EntityWarmlyLocomotive warmlyLocomotive)
            {
                warmlyLocomotive.AdditionalColor = (Color)e.Data.GetData(typeof(Color));
            }
            DrawLocomotive();
        }
        /// <summary>
        /// Добавление локомотива
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonOk_Click(object sender, EventArgs e)
        {
            EventAddLocomotive?.Invoke(_locomotive);
            Close();
        }
        /// <summary>
        /// Запоминаем объект от класса доп. прорисовки в переменную типа DataObject
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LabelOrnament_MouseDown(object sender, MouseEventArgs e)
        {
            IDrawningAdditionalElements selectedOrnament = new DrawningWheels();
            switch ((sender as Label).Name)
            {
                case "labelNoOrnament":
                    selectedOrnament = new DrawningWheels();
                    break;
                case "labelRectOrnament":
                    selectedOrnament = new DrawningRectOrnament();
                    break;
                case "labelEllipseOrnament":
                    selectedOrnament = new DrawningEllipseOrnament();
                    break;
            }
            var dataObj = new DataObject();
            dataObj.SetData(typeof(IDrawningAdditionalElements), selectedOrnament);
            (sender as Label).DoDragDrop(dataObj, DragDropEffects.Move | DragDropEffects.Copy);
        }
        /// <summary>
        /// Автоматически обновляем количество колёс
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void NumericUpDownWheelsNumber_ValueChanged(object sender, EventArgs e)
        {
            if (_locomotive != null)
            {
                _locomotive.AdditionalElements.WheelsNum = (int)numericUpDownWheelsNumber.Value;
                DrawLocomotive();
            }
        }
    }
}
