﻿namespace WarmlyLocomotove
{
    /// <summary>
    /// Класс для хранения коллекции карт
    /// </summary>
    internal class MapsCollection
    {
        /// <summary>
        /// Словарь (хранилище) с картами
        /// </summary>
        readonly Dictionary<string, MapWithSetLocomotivesGeneric<DrawningObjectLocomotive, AbstractMap>> _mapStorages;
        /// <summary>
        /// Возвращение списка названий карт
        /// </summary>
        public List<string> Keys => _mapStorages.Keys.ToList();
        /// <summary>
        /// Ширина окна отрисовки
        /// </summary>
        private readonly int _pictureWidth;
        /// <summary>
        /// Высота окна отрисовки
        /// </summary>
        private readonly int _pictureHeight;
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="pictureWidth"></param>
        /// <param name="pictureHeight"></param>
        public MapsCollection(int pictureWidth, int pictureHeight)
        {
            _mapStorages = new Dictionary<string, MapWithSetLocomotivesGeneric<DrawningObjectLocomotive, AbstractMap>>();
            _pictureWidth = pictureWidth;
            _pictureHeight = pictureHeight;
            _removedObjects = new LinkedList<DrawningObjectLocomotive>();
        }
        /// <summary>
        /// Добавление карты
        /// </summary>
        /// <param name="name">Название карты</param>
        /// <param name="map">Карта</param>
        public void AddMap(string name, AbstractMap map)
        {
            if (!_mapStorages.ContainsKey(name))
            {
                _mapStorages.Add(name, new MapWithSetLocomotivesGeneric<DrawningObjectLocomotive, AbstractMap>(_pictureWidth, _pictureHeight, map));
            }
        }
        /// <summary>
        /// Удаление карты
        /// </summary>
        /// <param name="name">Название карты</param>
        public void DelMap(string name)
        {
            _mapStorages.Remove(name);
        }
        /// <summary>
        /// Доступ к парковке
        /// </summary>
        /// <param name="ind"></param>
        /// <returns></returns>
        public MapWithSetLocomotivesGeneric<DrawningObjectLocomotive, AbstractMap> this[string ind]
        {
            get
            {
                if (_mapStorages.ContainsKey(ind))
                {
                    return _mapStorages[ind];
                }
                return null;
            }
        }
        /// <summary>
        /// Индексатор
        /// </summary>
        /// <param name="dictIndex">Индекс элемента словаря</param>
        /// <param name="objIndex">Индекс объекта в элементе словаря</param>
        /// <returns></returns>
        public IDrawningObject this[string dictIndex, int objIndex]
        {
            get
            {
                if (_mapStorages.ContainsKey(dictIndex))
                {
                    var selectedDictElement = _mapStorages[dictIndex];
                    var selectedObject = selectedDictElement.GetObject(objIndex);
                    return selectedObject;
                }
                return null;
            }
        }
        /// <summary>
        /// Коллекция удалённых объектов
        /// </summary>
        private LinkedList<DrawningObjectLocomotive> _removedObjects;
        /// <summary>
        /// Свойство для заполнения коллекции (добавляем в конец, и получаем из конца)
        /// </summary>
        public DrawningObjectLocomotive RemovedObject
        {
            set
            {
                _removedObjects.AddLast(value);
            }
            get
            {
                if (_removedObjects.Count == 0)
                {
                    return null;
                }
                var removedObject = _removedObjects.Last();
                return removedObject;
            }
        }
        public void RemoveLastObject()
        {
            _removedObjects.RemoveLast();
        }
    }

}
