﻿namespace WarmlyLocomotove
{
    internal class SetLocomotivesGeneric<T>
        where T : class
    {
        /// <summary>
        /// Массив объектов, которые храним
        /// </summary>
        private readonly List<T> _places;
        /// <summary>
        /// Количество объектов в списке
        /// </summary>
        public int Count => _places.Count;
        private readonly int _maxCount;
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="count"></param>
        public SetLocomotivesGeneric(int count)
        {
            _maxCount = count;
            _places = new List<T>();
        }
        /// <summary>
        /// Добавление объекта в набор на конкретную позицию
        /// </summary>
        /// <param name="locomotive">Добавляемый локомотив</param>
        /// <param name="position">Позиция</param>
        /// <returns></returns>
        public int Insert(T locomotive, int position)
        {
            if (position < 0 || position > Count + 1)
            {
                return position;
            }
            if (_places.Count == 0)
            {
                _places.Add(locomotive);
                return 0;
            }
            _places.Insert(position, locomotive);
            return position;
        }
        /// <summary>
        /// Удаление объекта из набора с конкретной позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T Remove(int position)
        {
            if (position > Count || _places[position] == null)
            {
                return null;
            }
            var result = _places[position];
            _places.RemoveAt(position);
            return result;
        }
        /// <summary>
        /// Получение объекта из набора по позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T this[int position]
        {
            get
            {
                if (_places[position] != null)
                {
                    return _places[position];
                }
                else
                {
                    return null;
                }
            }
            set
            {
                Insert(value, position);
            }
        }
        /// <summary>
        /// Проход по набору до первого пустого
        /// </summary>
        /// <returns></returns>
        public IEnumerable<T> GetLocomotives()
        {
            foreach (var locomotive in _places)
            {
                if (locomotive != null)
                {
                    yield return locomotive;
                }
                else
                {
                    yield break;
                }
            }
        }
    }
}
